@@ -24,30 +24,38 @@
 
 ## EV test based on Cn
 
-evTest <- function(x, r = 6, N = 1000, m = 30, offset=0.5)
+evTest <- function(x,N = 1000)
 {
-  nr <- length(r)
-  
   ## make pseudo-observations
   p <- ncol(x)
   n <- nrow(x)
   u <- apply(x,2,rank)/(n+1)
 
+  ## set r according to recommendations
+  if (p == 2)
+    r <- 5:7
+  else if (p == 3 || p == 4)
+    r <- 2:4
+  else if (p > 4)
+    {
+      warning("the test might not hold its level for small sample sizes")
+      r <- 2:3
+    }
+  else stop("the data matrix should have at least two columns")
+    
+  nr <- length(r)
+  
   ## make grid
-  if (m > 0)
-    {
-      y <- seq(1/m, 1 - 1/m, len = m)
-      v <- vector("list", p)
-      for (i in 1:p)
-        v[[i]] <- y
-      g <- as.matrix(expand.grid(v))
-      m <- nrow(g)
-    }
-  else 
-    {
-      g <- u ## pseudo-observations
-      m <- n
-    }
+  m <- ceiling(2000^(1/p)) # grid size
+  y <- seq(1/m, 1 - 1/m, len = m)
+  v <- vector("list", p)
+  for (i in 1:p)
+    v[[i]] <- y
+  g <- as.matrix(expand.grid(v))
+  m <- nrow(g)
+  
+  ## not very important
+  offset <- 0.5
 
   ## compute the test statistic
   s <- .C("evtest_stat",
@@ -75,15 +83,25 @@ evTest <- function(x, r = 6, N = 1000, m = 30, offset=0.5)
            s0 = double(N * nr),
            PACKAGE="copula")$s0
 
-  ## combined p-values?
+  ## p-values
   s0 <- matrix(s0, ncol = nr, byrow = TRUE)
-  pval <- apply(s0 >= matrix(s, nrow=N, ncol=nr, byrow=TRUE),
-                2, function(x) (sum(x) + 0.5) / (N + 1) )
-  comb.s <- sum(s) / nr
-  comb.pval <- ( sum( apply(s0, 1, sum) / nr >= comb.s) + 0.5 ) / (N + 1) 
+  #pval <- apply(s0 >= matrix(s, nrow=N, ncol=nr, byrow=TRUE),
+  #              2, function(x) (sum(x) + 0.5) / (N + 1) )
+  comb.s <- sum(s) 
+  comb.pval <- ( sum( apply(s0, 1, sum) >= comb.s ) + 0.5 ) / (N + 1) 
 
-  return(list(statistic=c(s, comb.s),
-              pvalue=c(pval, comb.pval), s0=s0))
+  #return(list(statistic=c(s, comb.s),
+  #            pvalue=c(pval, comb.pval), s0=s0))
+
+  evt <- list(statistic=comb.s,pvalue=comb.pval)
+  class(evt) <- "evTest"
+  evt
+}
+
+print.evTest <- function(x, ...)
+{
+  cat("Cramer-von Mises statistic:", x$statistic,
+      "with p-value", x$pvalue, "\n\n")
 }
 
 ###################################################################
